#include "config.h"
#include <stdlib.h>
#include <unistd.h>
#include "fdbuf.h"
#include "itoa.h"
#include "mystring.h"
#include "hostname.h"

const char* filename = 0;

int smtp_get(mystring& str)
{
  mystring tmp;
  str = "";
  int code = -1;
  while(fin.getline(tmp)) {
    if(tmp[tmp.length()-1] == '\r')
      tmp = tmp.left(tmp.length()-1);
    code = atoi(tmp.c_str());
    if(!!str)
      str += "/";
    str += tmp.right(4);
    if(tmp[3] != '-')
      break;
  }
  return code;
}

int smtp_put(mystring cmd, mystring& result)
{
  if(!fout.write(cmd) ||
     !fout.write("\r\n") ||
     !fout.flush())
    return -1;
  return smtp_get(result);
}

void smtp_do(mystring cmd, int range)
{
  mystring msg;
  int code;
  if(!cmd)
    code = smtp_get(msg);
  else
    code = smtp_put(cmd, msg);
  if(code < range || code >= (range+100)) {
    ferr.write(filename);
    ferr.write(": SMTP Error: ");
    ferr.write(itoa(code));
    ferr.write(" ");
    ferr.writeln(msg);
    exit(1);
  }
}

bool send_envelope(fdibuf& msg)
{
  mystring tmp;
  msg.getline(tmp);
  smtp_do("MAIL FROM: <" + tmp + ">", 250);
  while(msg.getline(tmp) && !!tmp) {
    smtp_do("RCPT TO: <" + tmp + ">", 250);
  }
  return true;
}

bool send_data(fdibuf& msg)
{
  smtp_do("DATA", 354);
  mystring tmp;
  while(msg.getline(tmp)) {
    if(!fout.write(tmp) ||
       !fout.write("\r\n")) {
      ferr.write(filename);
      ferr.writeln(" SMTP error while writing message");
      return false;
    }
  }
  smtp_do(".", 250);
  return true;
}

int main(int argc, char* argv[])
{
  smtp_do("", 220);
  smtp_do("HELO " + hostname(), 250);
  for(int i = 1; i < argc; i++) {
    filename = argv[i];
    fdibuf msg(filename);
    if(!msg) {
      ferr.write(filename);
      ferr.writeln("  Could not open");
      return 1;
    }
    if(!send_envelope(msg) ||
       !send_data(msg))
      return 1;
    ferr.write(filename);
    ferr.writeln(" OK");
  }
  smtp_do("QUIT", 221);
  return 0;
}
